from openapi_spec_validator.versions.consts import OPENAPIV2
from openapi_spec_validator.versions.consts import OPENAPIV30
from openapi_spec_validator.versions.consts import OPENAPIV31
from openapi_spec_validator.versions.datatypes import SpecVersion
from openapi_spec_validator.versions.shortcuts import get_spec_version

__all__ = [
    "OPENAPIV2",
    "OPENAPIV30",
    "OPENAPIV31",
    "SpecVersion",
    "get_spec_version",
]



from openapi_spec_validator import OpenAPIV2SpecValidator
from openapi_spec_validator import OpenAPIV30SpecValidator
from openapi_spec_validator.validation.exceptions import (
    DuplicateOperationIDError,
)
from openapi_spec_validator.validation.exceptions import ExtraParametersError
from openapi_spec_validator.validation.exceptions import OpenAPIValidationError
from openapi_spec_validator.validation.exceptions import (
    UnresolvableParameterError,
)


class TestSpecValidatorIterErrors:
    def test_empty(self):
        spec = {}

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert errors_list[0].__class__ == OpenAPIValidationError
        assert errors_list[0].message == "'openapi' is a required property"
        assert errors_list[1].__class__ == OpenAPIValidationError
        assert errors_list[1].message == "'info' is a required property"
        assert errors_list[2].__class__ == OpenAPIValidationError
        assert errors_list[2].message == "'paths' is a required property"

    def test_info_empty(self):
        spec = {
            "openapi": "3.0.0",
            "info": {},
            "paths": {},
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert errors_list[0].__class__ == OpenAPIValidationError
        assert errors_list[0].message == "'title' is a required property"

    def test_minimalistic(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {},
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert errors_list == []

    def test_same_parameters_names(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {
                "/test/{param1}": {
                    "parameters": [
                        {
                            "name": "param1",
                            "in": "query",
                            "schema": {
                                "type": "integer",
                            },
                        },
                        {
                            "name": "param1",
                            "in": "path",
                            "schema": {
                                "type": "integer",
                            },
                            "required": True,
                        },
                    ],
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert errors_list == []

    def test_same_operation_ids(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {
                "/test": {
                    "get": {
                        "operationId": "operation1",
                        "responses": {
                            "default": {
                                "description": "default response",
                            },
                        },
                    },
                    "post": {
                        "operationId": "operation1",
                        "responses": {
                            "default": {
                                "description": "default response",
                            },
                        },
                    },
                },
                "/test2": {
                    "get": {
                        "operationId": "operation1",
                        "responses": {
                            "default": {
                                "description": "default response",
                            },
                        },
                    },
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert len(errors_list) == 2
        assert errors_list[0].__class__ == DuplicateOperationIDError
        assert errors_list[1].__class__ == DuplicateOperationIDError

    def test_allow_allof_required_no_properties(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {},
            "components": {
                "schemas": {
                    "Credit": {
                        "type": "object",
                        "properties": {
                            "clientId": {"type": "string"},
                        },
                    },
                    "CreditCreate": {
                        "allOf": [
                            {"$ref": "#/components/schemas/Credit"},
                            {"required": ["clientId"]},
                        ]
                    },
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()
        errors_list = list(errors)
        assert errors_list == []

    def test_allow_allof_when_required_is_linked_to_the_parent_object(self):
        spec = {
            "openapi": "3.0.1",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {},
            "components": {
                "schemas": {
                    "Address": {
                        "type": "object",
                        "properties": {
                            "SubdivisionCode": {
                                "type": "string",
                                "description": "State or region",
                            },
                            "Town": {
                                "type": "string",
                                "description": "Town or city",
                            },
                            "CountryCode": {
                                "type": "string",
                            },
                        },
                    },
                    "AddressCreation": {
                        "required": ["CountryCode", "Town"],
                        "type": "object",
                        "allOf": [{"$ref": "#/components/schemas/Address"}],
                    },
                }
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()
        errors_list = list(errors)
        assert errors_list == []

    def test_extra_parameters_in_required(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {},
            "components": {
                "schemas": {
                    "testSchema": {
                        "type": "object",
                        "required": [
                            "testparam1",
                        ],
                    }
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert errors_list[0].__class__ == ExtraParametersError
        assert errors_list[0].message == (
            "Required list has not defined properties: ['testparam1']"
        )

    def test_undocumented_parameter(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {
                "/test/{param1}/{param2}": {
                    "get": {
                        "responses": {
                            "default": {
                                "description": "default response",
                            },
                        },
                    },
                    "parameters": [
                        {
                            "name": "param1",
                            "in": "path",
                            "schema": {
                                "type": "integer",
                            },
                            "required": True,
                        },
                    ],
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert errors_list[0].__class__ == UnresolvableParameterError
        assert errors_list[0].message == (
            "Path parameter 'param2' for 'get' operation in "
            "'/test/{param1}/{param2}' was not resolved"
        )

    def test_default_value_wrong_type(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {},
            "components": {
                "schemas": {
                    "test": {
                        "type": "integer",
                        "default": "invaldtype",
                    },
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert len(errors_list) == 1
        assert errors_list[0].__class__ == OpenAPIValidationError
        assert errors_list[0].message == (
            "'invaldtype' is not of type 'integer'"
        )

    def test_parameter_default_value_wrong_type(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {
                "/test/{param1}": {
                    "get": {
                        "responses": {
                            "default": {
                                "description": "default response",
                            },
                        },
                    },
                    "parameters": [
                        {
                            "name": "param1",
                            "in": "path",
                            "schema": {
                                "type": "integer",
                                "default": "invaldtype",
                            },
                            "required": True,
                        },
                    ],
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert len(errors_list) == 1
        assert errors_list[0].__class__ == OpenAPIValidationError
        assert errors_list[0].message == (
            "'invaldtype' is not of type 'integer'"
        )

    def test_parameter_default_value_wrong_type_swagger(self):
        spec = {
            "swagger": "2.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {
                "/test/": {
                    "get": {
                        "responses": {
                            "200": {
                                "description": "OK",
                                "schema": {"type": "object"},
                            },
                        },
                        "parameters": [
                            {
                                "name": "param1",
                                "in": "query",
                                "type": "integer",
                                "default": "invaldtype",
                            },
                        ],
                    },
                },
            },
        }

        errors = OpenAPIV2SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert len(errors_list) == 1
        assert errors_list[0].__class__ == OpenAPIValidationError
        assert errors_list[0].message == (
            "'invaldtype' is not of type 'integer'"
        )

    def test_parameter_default_value_with_reference(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {
                "/test/": {
                    "get": {
                        "responses": {
                            "default": {
                                "description": "default response",
                            },
                        },
                        "parameters": [
                            {
                                "name": "param1",
                                "in": "query",
                                "schema": {
                                    "allOf": [
                                        {
                                            "$ref": "#/components/schemas/type",
                                        }
                                    ],
                                    "default": 1,
                                },
                            },
                        ],
                    },
                },
            },
            "components": {
                "schemas": {
                    "type": {
                        "type": "integer",
                    }
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert errors_list == []

    def test_parameter_custom_format_checker_not_found(self):
        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {
                "/test/": {
                    "get": {
                        "responses": {
                            "default": {
                                "description": "default response",
                            },
                        },
                        "parameters": [
                            {
                                "name": "param1",
                                "in": "query",
                                "schema": {
                                    "type": "string",
                                    "format": "custom",
                                    "default": "customvalue",
                                },
                            },
                        ],
                    },
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert errors_list == []

    def test_parameter_default_value_custom_format_invalid(self):
        from openapi_schema_validator import oas30_format_checker

        @oas30_format_checker.checks("custom")
        def validate(to_validate) -> bool:
            return to_validate == "valid"

        spec = {
            "openapi": "3.0.0",
            "info": {
                "title": "Test Api",
                "version": "0.0.1",
            },
            "paths": {
                "/test/": {
                    "get": {
                        "responses": {
                            "default": {
                                "description": "default response",
                            },
                        },
                        "parameters": [
                            {
                                "name": "param1",
                                "in": "query",
                                "schema": {
                                    "type": "string",
                                    "format": "custom",
                                    "default": "invalid",
                                },
                            },
                        ],
                    },
                },
            },
        }

        errors = OpenAPIV30SpecValidator(spec).iter_errors()

        errors_list = list(errors)
        assert len(errors_list) == 1
        assert errors_list[0].__class__ == OpenAPIValidationError
        assert errors_list[0].message == ("'invalid' is not a 'custom'")



import pytest

from openapi_spec_validator.versions import consts as versions
from openapi_spec_validator.versions.exceptions import OpenAPIVersionNotFound
from openapi_spec_validator.versions.shortcuts import get_spec_version


class TestGetSpecVersion:
    def test_no_keyword(self):
        spec = {}

        with pytest.raises(OpenAPIVersionNotFound):
            get_spec_version(spec)

    @pytest.mark.parametrize("keyword", ["swagger", "openapi"])
    @pytest.mark.parametrize("version", ["x.y.z", "xyz2.0.0", "2.xyz0.0"])
    def test_invalid(self, keyword, version):
        spec = {
            keyword: version,
        }

        with pytest.raises(OpenAPIVersionNotFound):
            get_spec_version(spec)

    @pytest.mark.parametrize(
        "keyword,version,expected",
        [
            ("swagger", "2.0", versions.OPENAPIV2),
            ("openapi", "3.0.0", versions.OPENAPIV30),
            ("openapi", "3.0.1", versions.OPENAPIV30),
            ("openapi", "3.0.2", versions.OPENAPIV30),
            ("openapi", "3.0.3", versions.OPENAPIV30),
            ("openapi", "3.1.0", versions.OPENAPIV31),
        ],
    )
    def test_valid(self, keyword, version, expected):
        spec = {
            keyword: version,
        }

        result = get_spec_version(spec)

        assert result == expected



from typing import List

from openapi_spec_validator.versions.datatypes import SpecVersion

OPENAPIV2 = SpecVersion(
    keyword="swagger",
    major="2",
    minor="0",
)

OPENAPIV30 = SpecVersion(
    keyword="openapi",
    major="3",
    minor="0",
)

OPENAPIV31 = SpecVersion(
    keyword="openapi",
    major="3",
    minor="1",
)

VERSIONS: List[SpecVersion] = [OPENAPIV2, OPENAPIV30, OPENAPIV31]



from os import path
from pathlib import PurePath
from urllib.parse import urlunparse

import pytest
from jsonschema_path.handlers.file import FilePathHandler
from jsonschema_path.handlers.urllib import UrllibHandler


def spec_file_url(spec_file, schema="file"):
    directory = path.abspath(path.dirname(__file__))
    full_path = path.join(directory, spec_file)
    return urlunparse((schema, None, full_path, None, None, None))


def spec_from_file(spec_file):
    directory = path.abspath(path.dirname(__file__))
    path_full = path.join(directory, spec_file)
    uri = PurePath(path_full).as_uri()
    return FilePathHandler()(uri)


def spec_from_url(spec_url):
    return UrllibHandler("http", "https")(spec_url)


class Factory(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__


@pytest.fixture
def factory():
    return Factory(
        spec_file_url=spec_file_url,
        spec_from_file=spec_from_file,
        spec_from_url=spec_from_url,
    )



import logging
import sys
from argparse import ArgumentParser
from typing import Optional
from typing import Sequence

from jsonschema.exceptions import ValidationError
from jsonschema.exceptions import best_match

from openapi_spec_validator.readers import read_from_filename
from openapi_spec_validator.readers import read_from_stdin
from openapi_spec_validator.shortcuts import validate
from openapi_spec_validator.validation import OpenAPIV2SpecValidator
from openapi_spec_validator.validation import OpenAPIV30SpecValidator
from openapi_spec_validator.validation import OpenAPIV31SpecValidator

logger = logging.getLogger(__name__)
logging.basicConfig(
    format="%(asctime)s %(levelname)s %(name)s %(message)s",
    level=logging.WARNING,
)


def print_ok(filename: str) -> None:
    print(f"{filename}: OK")


def print_error(filename: str, exc: Exception) -> None:
    print(f"{filename}: Error: {exc}")


def print_validationerror(
    filename: str, exc: ValidationError, errors: str = "best-match"
) -> None:
    print(f"{filename}: Validation Error: {exc}")
    if exc.cause:
        print("\n# Cause\n")
        print(exc.cause)
    if not exc.context:
        return
    if errors == "all":
        print("\n\n# Due to one of those errors\n")
        print("\n\n\n".join("## " + str(e) for e in exc.context))
    elif errors == "best-match":
        print("\n\n# Probably due to this subschema error\n")
        print("## " + str(best_match(exc.context)))
        if len(exc.context) > 1:
            print(
                f"\n({len(exc.context) - 1} more subschemas errors,",
                "use --errors=all to see them.)",
            )


def main(args: Optional[Sequence[str]] = None) -> None:
    parser = ArgumentParser()
    parser.add_argument(
        "file",
        nargs="+",
        help="Validate specified file(s).",
    )
    parser.add_argument(
        "--errors",
        choices=("best-match", "all"),
        default="best-match",
        help="""Control error reporting. Defaults to "best-match", """
        """use "all" to get all subschema errors.""",
    )
    parser.add_argument(
        "--schema",
        type=str,
        choices=["detect", "2.0", "3.0", "3.1", "3.0.0", "3.1.0"],
        default="detect",
        metavar="{detect,2.0,3.0,3.1}",
        help="OpenAPI schema version (default: detect).",
    )
    args_parsed = parser.parse_args(args)

    for filename in args_parsed.file:
        # choose source
        reader = read_from_filename
        if filename in ["-", "/-"]:
            filename = "stdin"
            reader = read_from_stdin

        # read source
        try:
            spec, base_uri = reader(filename)
        except Exception as exc:
            print(exc)
            sys.exit(1)

        # choose the validator
        validators = {
            "detect": None,
            "2.0": OpenAPIV2SpecValidator,
            "3.0": OpenAPIV30SpecValidator,
            "3.1": OpenAPIV31SpecValidator,
            # backward compatibility
            "3.0.0": OpenAPIV30SpecValidator,
            "3.1.0": OpenAPIV31SpecValidator,
        }
        validator_cls = validators[args_parsed.schema]

        # validate
        try:
            validate(spec, base_uri=base_uri, cls=validator_cls)
        except ValidationError as exc:
            print_validationerror(filename, exc, args_parsed.errors)
            sys.exit(1)
        except Exception as exc:
            print_error(filename, exc)
            sys.exit(2)
        else:
            print_ok(filename)


if __name__ == "__main__":
    main()



"""OpenAPI spec validator module."""
from openapi_spec_validator.shortcuts import validate
from openapi_spec_validator.shortcuts import validate_spec
from openapi_spec_validator.shortcuts import validate_spec_url
from openapi_spec_validator.shortcuts import validate_url
from openapi_spec_validator.validation import OpenAPIV2SpecValidator
from openapi_spec_validator.validation import OpenAPIV3SpecValidator
from openapi_spec_validator.validation import OpenAPIV30SpecValidator
from openapi_spec_validator.validation import OpenAPIV31SpecValidator
from openapi_spec_validator.validation import openapi_v2_spec_validator
from openapi_spec_validator.validation import openapi_v3_spec_validator
from openapi_spec_validator.validation import openapi_v30_spec_validator
from openapi_spec_validator.validation import openapi_v31_spec_validator

__author__ = "Artur Maciag"
__email__ = "maciag.artur@gmail.com"
__version__ = "0.7.1"
__url__ = "https://github.com/python-openapi/openapi-spec-validator"
__license__ = "Apache License, Version 2.0"

__all__ = [
    "openapi_v2_spec_validator",
    "openapi_v3_spec_validator",
    "openapi_v30_spec_validator",
    "openapi_v31_spec_validator",
    "OpenAPIV2SpecValidator",
    "OpenAPIV3SpecValidator",
    "OpenAPIV30SpecValidator",
    "OpenAPIV31SpecValidator",
    "validate",
    "validate_url",
    "validate_spec",
    "validate_spec_url",
]



"""OpenAPI spec validator shortcuts module."""
import warnings
from typing import Mapping
from typing import Optional
from typing import Type

from jsonschema_path import SchemaPath
from jsonschema_path.handlers import all_urls_handler
from jsonschema_path.typing import Schema

from openapi_spec_validator.validation import OpenAPIV2SpecValidator
from openapi_spec_validator.validation import OpenAPIV30SpecValidator
from openapi_spec_validator.validation import OpenAPIV31SpecValidator
from openapi_spec_validator.validation.exceptions import ValidatorDetectError
from openapi_spec_validator.validation.protocols import SupportsValidation
from openapi_spec_validator.validation.types import SpecValidatorType
from openapi_spec_validator.validation.validators import SpecValidator
from openapi_spec_validator.versions import consts as versions
from openapi_spec_validator.versions.datatypes import SpecVersion
from openapi_spec_validator.versions.exceptions import OpenAPIVersionNotFound
from openapi_spec_validator.versions.shortcuts import get_spec_version

SPEC2VALIDATOR: Mapping[SpecVersion, SpecValidatorType] = {
    versions.OPENAPIV2: OpenAPIV2SpecValidator,
    versions.OPENAPIV30: OpenAPIV30SpecValidator,
    versions.OPENAPIV31: OpenAPIV31SpecValidator,
}


def get_validator_cls(spec: Schema) -> SpecValidatorType:
    try:
        spec_version = get_spec_version(spec)
    # backward compatibility
    except OpenAPIVersionNotFound:
        raise ValidatorDetectError
    return SPEC2VALIDATOR[spec_version]


def validate(
    spec: Schema,
    base_uri: str = "",
    cls: Optional[SpecValidatorType] = None,
) -> None:
    if cls is None:
        cls = get_validator_cls(spec)
    sp = SchemaPath.from_dict(spec, base_uri=base_uri)
    v = cls(sp)
    return v.validate()


def validate_url(
    spec_url: str,
    cls: Optional[Type[SpecValidator]] = None,
) -> None:
    spec = all_urls_handler(spec_url)
    return validate(spec, base_uri=spec_url, cls=cls)


def validate_spec(
    spec: Schema,
    base_uri: str = "",
    validator: Optional[SupportsValidation] = None,
    cls: Optional[SpecValidatorType] = None,
    spec_url: Optional[str] = None,
) -> None:
    warnings.warn(
        "validate_spec shortcut is deprecated. Use validate instead.",
        DeprecationWarning,
    )
    if validator is not None:
        warnings.warn(
            "validator parameter is deprecated. Use cls instead.",
            DeprecationWarning,
        )
        return validator.validate(spec, base_uri=base_uri, spec_url=spec_url)
    if cls is None:
        cls = get_validator_cls(spec)
    v = cls(spec)
    return v.validate()


def validate_spec_url(
    spec_url: str,
    validator: Optional[SupportsValidation] = None,
    cls: Optional[Type[SpecValidator]] = None,
) -> None:
    warnings.warn(
        "validate_spec_url shortcut is deprecated. Use validate_url instead.",
        DeprecationWarning,
    )
    if validator is not None:
        warnings.warn(
            "validator parameter is deprecated. Use cls instead.",
            DeprecationWarning,
        )
        spec = all_urls_handler(spec_url)
        return validator.validate(spec, base_uri=spec_url)
    return validate_url(spec_url, cls=cls)



import pytest

from openapi_spec_validator import OpenAPIV2SpecValidator
from openapi_spec_validator import OpenAPIV30SpecValidator
from openapi_spec_validator import openapi_v2_spec_validator
from openapi_spec_validator import openapi_v30_spec_validator
from openapi_spec_validator import validate
from openapi_spec_validator import validate_spec
from openapi_spec_validator import validate_spec_url
from openapi_spec_validator import validate_url
from openapi_spec_validator.validation.exceptions import OpenAPIValidationError
from openapi_spec_validator.validation.exceptions import ValidatorDetectError


class TestValidateSpec:
    def test_spec_schema_version_not_detected(self):
        spec = {}

        with pytest.raises(ValidatorDetectError):
            validate(spec)


class TestLocalValidateSpecUrl:
    def test_spec_schema_version_not_detected(self, factory):
        spec_path = "data/empty.yaml"
        spec_url = factory.spec_file_url(spec_path)

        with pytest.raises(ValidatorDetectError):
            validate_url(spec_url)


class TestLiocalValidatev2Spec:
    LOCAL_SOURCE_DIRECTORY = "data/v2.0/"

    def local_test_suite_file_path(self, test_file):
        return f"{self.LOCAL_SOURCE_DIRECTORY}{test_file}"

    @pytest.mark.parametrize(
        "spec_file",
        [
            "petstore.yaml",
        ],
    )
    def test_valid(self, factory, spec_file):
        spec_path = self.local_test_suite_file_path(spec_file)
        spec = factory.spec_from_file(spec_path)

        validate(spec)
        validate(spec, cls=OpenAPIV2SpecValidator)
        with pytest.warns(DeprecationWarning):
            validate_spec(spec, validator=openapi_v2_spec_validator)

    @pytest.mark.parametrize(
        "spec_file",
        [
            "empty.yaml",
        ],
    )
    def test_falied(self, factory, spec_file):
        spec_path = self.local_test_suite_file_path(spec_file)
        spec = factory.spec_from_file(spec_path)

        with pytest.raises(OpenAPIValidationError):
            validate(spec, cls=OpenAPIV2SpecValidator)
        with pytest.warns(DeprecationWarning):
            with pytest.raises(OpenAPIValidationError):
                validate_spec(spec, validator=openapi_v2_spec_validator)


class TestLocalValidatev30Spec:
    LOCAL_SOURCE_DIRECTORY = "data/v3.0/"

    def local_test_suite_file_path(self, test_file):
        return f"{self.LOCAL_SOURCE_DIRECTORY}{test_file}"

    @pytest.mark.parametrize(
        "spec_file",
        [
            "petstore.yaml",
        ],
    )
    def test_valid(self, factory, spec_file):
        spec_path = self.local_test_suite_file_path(spec_file)
        spec = factory.spec_from_file(spec_path)
        spec_url = factory.spec_file_url(spec_path)

        validate(spec)
        with pytest.warns(DeprecationWarning):
            validate_spec(spec, spec_url=spec_url)
        validate(spec, cls=OpenAPIV30SpecValidator)
        with pytest.warns(DeprecationWarning):
            validate_spec(spec, validator=openapi_v30_spec_validator)

    @pytest.mark.parametrize(
        "spec_file",
        [
            "empty.yaml",
        ],
    )
    def test_falied(self, factory, spec_file):
        spec_path = self.local_test_suite_file_path(spec_file)
        spec = factory.spec_from_file(spec_path)

        with pytest.raises(OpenAPIValidationError):
            validate(spec, cls=OpenAPIV30SpecValidator)
        with pytest.warns(DeprecationWarning):
            with pytest.raises(OpenAPIValidationError):
                validate_spec(spec, validator=openapi_v30_spec_validator)


@pytest.mark.network
class TestRemoteValidatev2SpecUrl:
    REMOTE_SOURCE_URL = (
        "https://raw.githubusercontent.com/OAI/OpenAPI-Specification/"
    )

    def remote_test_suite_file_path(self, test_file):
        return f"{self.REMOTE_SOURCE_URL}{test_file}"

    @pytest.mark.parametrize(
        "spec_file",
        [
            "f25a1d44cff9669703257173e562376cc5bd0ec6/examples/v2.0/"
            "yaml/petstore.yaml",
            "f25a1d44cff9669703257173e562376cc5bd0ec6/examples/v2.0/"
            "yaml/api-with-examples.yaml",
            "f25a1d44cff9669703257173e562376cc5bd0ec6/examples/v2.0/"
            "yaml/petstore-expanded.yaml",
        ],
    )
    def test_valid(self, spec_file):
        spec_url = self.remote_test_suite_file_path(spec_file)

        validate_url(spec_url)
        validate_url(spec_url, cls=OpenAPIV2SpecValidator)
        with pytest.warns(DeprecationWarning):
            validate_spec_url(spec_url)
            validate_spec_url(spec_url, validator=openapi_v2_spec_validator)


@pytest.mark.network
class TestRemoteValidatev30SpecUrl:
    REMOTE_SOURCE_URL = (
        "https://raw.githubusercontent.com/OAI/OpenAPI-Specification/"
    )

    def remote_test_suite_file_path(self, test_file):
        return f"{self.REMOTE_SOURCE_URL}{test_file}"

    @pytest.mark.parametrize(
        "spec_file",
        [
            "f75f8486a1aae1a7ceef92fbc63692cb2556c0cd/examples/v3.0/"
            "petstore.yaml",
            "f75f8486a1aae1a7ceef92fbc63692cb2556c0cd/examples/v3.0/"
            "api-with-examples.yaml",
            "970566d5ca236a5ce1a02fb7d617fdbd07df88db/examples/v3.0/"
            "api-with-examples.yaml",
        ],
    )
    def test_valid(self, spec_file):
        spec_url = self.remote_test_suite_file_path(spec_file)

        validate_url(spec_url)
        validate_url(spec_url, cls=OpenAPIV30SpecValidator)
        with pytest.warns(DeprecationWarning):
            validate_spec_url(spec_url, validator=openapi_v30_spec_validator)



from re import compile
from typing import List

from jsonschema_path.typing import Schema

from openapi_spec_validator.versions.datatypes import SpecVersion
from openapi_spec_validator.versions.exceptions import OpenAPIVersionNotFound


class SpecVersionFinder:
    pattern = compile(r"(?P<major>\d+)\.(?P<minor>\d+)(\..*)?")

    def __init__(self, versions: List[SpecVersion]) -> None:
        self.versions = versions

    def find(self, spec: Schema) -> SpecVersion:
        for v in self.versions:
            if v.keyword in spec:
                version_str = spec[v.keyword]
                m = self.pattern.match(version_str)
                if m:
                    version = SpecVersion(**m.groupdict(), keyword=v.keyword)
                    if v == version:
                        return v

        raise OpenAPIVersionNotFound
